
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "lafnames.h"
#include LA_VECTOR_DOUBLE_H
#include LA_SYMM_MAT_DOUBLE_H
#include LA_UNIT_UPPER_TRIANG_MAT_DOUBLE_H
#include LA_UPPER_TRIANG_MAT_DOUBLE_H
#include LA_UNIT_LOWER_TRIANG_MAT_DOUBLE_H
#include LA_LOWER_TRIANG_MAT_DOUBLE_H
#include LA_SPD_MAT_DOUBLE_H
#include LA_SYMM_BAND_MAT_DOUBLE_H
#include LA_TRIDIAG_MAT_DOUBLE_H

#ifdef LA_COMPLEX_SUPPORT
#include "f2c.h"
#include "lapackc.h"
#include LA_VECTOR_COMPLEX_H
#include LA_GEN_MAT_COMPLEX_H
#endif

#include "blas3pp.h"
#include "blas1.h"
#include "blas1pp.h" // for Blas_Norm2
#include "laindex.h"

void Blas_Mat_Mat_Mult(const LaGenMatDouble &A, 
            const LaGenMatDouble &B, LaGenMatDouble &C, 
            double alpha , double beta )
{
        char t = 'N';
        integer m = A.size(0), k = A.size(1), n = B.size(1);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(0) == C.size(0));
	assert(B.size(1) == C.size(1));
	assert(A.size(1) == B.size(0));

  F77NAME(dgemm)(&t, &t, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}

void Blas_Mat_Trans_Mat_Mult(const LaGenMatDouble &A, 
            const LaGenMatDouble &B, LaGenMatDouble &C, 
            double alpha, double beta )
{
        char nt = 'N';
        char t = 'T';
	// m, n, k have to be determined according to op(A), op(B)!
        integer m = A.size(1), k = A.size(0), n = B.size(1);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(1) == C.size(0));
	assert(B.size(1) == C.size(1));
	assert(A.size(0) == B.size(0));

  F77NAME(dgemm)(&t, &nt, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}

void Blas_Mat_Mat_Trans_Mult(const LaGenMatDouble &A, 
            const LaGenMatDouble &B, LaGenMatDouble &C, 
            double alpha,  double beta )
{
        char nt = 'N';
        char t = 'T';
	// m, n, k have to be determined according to op(A), op(B)!
        integer m = A.size(0), k = A.size(1), n = B.size(0);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(0) == C.size(0));
	assert(B.size(0) == C.size(1));
	assert(A.size(1) == B.size(1));

  F77NAME(dgemm)(&nt, &t, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}



double my_Dot_Prod(const LaGenMatDouble &dx, const LaGenMatDouble &dy)
{
  assert(dx.size(0)*dx.size(1)==dy.size(0)*dy.size(1));
  integer n = dx.size(0)*dx.size(1);
  integer incx = dx.inc(0)*dx.inc(1);
  integer incy = dy.inc(0)*dy.inc(1);

  return F77NAME(ddot)(&n, &dx(0,0), &incx, &dy(0,0), &incy);
}

void Blas_Mat_Mat_Mult(const LaGenMatDouble &A, 
		       const LaGenMatDouble &B, LaVectorDouble &C)
{
  // calculate only the diagonal of A times B
  int msize = std::min(A.size(0), B.size(1));
  assert(A.size(1) == B.size(0));
  assert(C.size() >= msize);
  for (int i=0; i < msize; i++)
    C(i) = my_Dot_Prod(A(LaIndex(i), LaIndex(0,A.size(1))),
		       // FIXME: or is the index rather LaIndex(0,A.size(1)-1) ?
		       B(LaIndex(0,B.size(0)), LaIndex(i)));
}

void Blas_Mat_Trans_Mat_Mult(const LaGenMatDouble &A, 
			     const LaGenMatDouble &B, LaVectorDouble &C)
{
  // calculate only the diagonal of A times B
  int msize = std::min(A.size(0), B.size(1));
  assert(A.size(0) == B.size(0));
  assert(C.size() >= msize);
  for (int i=0; i < msize; i++)
    C(i) = my_Dot_Prod(A(LaIndex(0,A.size(0)), LaIndex(i)),
		       B(LaIndex(0,B.size(0)), LaIndex(i)));
}

void Blas_Mat_Mat_Trans_Mult(const LaGenMatDouble &A, 
			     const LaGenMatDouble &B, LaVectorDouble &C)
{
  // calculate only the diagonal of A times B
  int msize = std::min(A.size(0), B.size(1));
  assert(A.size(1) == B.size(1));
  assert(C.size() >= msize);
  for (int i=0; i < msize; i++)
    C(i) = my_Dot_Prod(A(LaIndex(i), LaIndex(0,A.size(1))),
		       B(LaIndex(i), LaIndex(0,B.size(1))));
}






#ifdef _LA_UNIT_LOWER_TRIANG_MAT_DOUBLE_H_

void Blas_Mat_Mat_Solve(LaUnitLowerTriangMatDouble &A, 
            LaGenMatDouble &B, double alpha)
{
        char side = 'L', uplo = 'L', transa = 'N', diag = 'U';
        integer m = B.size(0), n = B.size(1), 
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha, 
                &A(0,0), &lda, &B(0,0), &ldb);
}

#endif

#ifdef _LA_UNIT_UPPER_TRIANG_MAT_DOUBLE_H_
void Blas_Mat_Mat_Mult(LaUnitUpperTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'N', diag = 'U';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrmm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}

void Blas_Mat_Mat_Solve(LaUnitUpperTriangMatDouble &A, 
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'N', diag = 'U';
        integer m = B.size(0), n = B.size(1), 
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha, 
                &A(0,0), &lda, &B(0,0), &ldb);
}

#endif

#ifdef _LA_LOWER_TRIANG_MAT_DOUBLE_H_
void Blas_Mat_Mat_Mult(LaLowerTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'L', transa = 'N', diag = 'N';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrmm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}

void Blas_Mat_Mat_Solve(LaLowerTriangMatDouble &A, 
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'L', transa = 'N', diag = 'N';
        integer m = B.size(0), n = B.size(1), 
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha, 
                &A(0,0), &lda, &B(0,0), &ldb);
}
#endif


#ifdef _LA_UPPER_TRIANG_MAT_DOUBLE_H_
void Blas_Mat_Mat_Mult(LaUpperTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'N', diag = 'N';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrmm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}

void Blas_Mat_Mat_Solve(LaUpperTriangMatDouble &A, 
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'N', diag = 'N';
        integer m = B.size(0), n = B.size(1), 
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha, 
                &A(0,0), &lda, &B(0,0), &ldb);
}
#endif


#ifdef _LA_UNIT_LOWER_TRIANG_MAT_DOUBLE_H_
void Blas_Mat_Trans_Mat_Solve(LaUnitLowerTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'L', transa = 'T', diag = 'U';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}
#endif

#ifdef _LA_UNIT_UPPER_TRIANG_MAT_DOUBLE_H_
void Blas_Mat_Trans_Mat_Solve(LaUnitUpperTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'T', diag = 'U';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}
#endif

#ifdef LA_LOWER_TRIANG_MAT_DOUBLE_H
void Blas_Mat_Mat_Mult(LaUnitLowerTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'L', transa = 'N', diag = 'U';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrmm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}

void Blas_Mat_Trans_Mat_Solve(LaLowerTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'L', transa = 'T', diag = 'N';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}
#endif


#ifdef _LA_UPPER_TRIANG_MAT_DOUBLE_H 
void Blas_Mat_Trans_Mat_Solve(LaUpperTriangMatDouble &A,
            LaGenMatDouble &B, double alpha )
{
        char side = 'L', uplo = 'U', transa = 'T', diag = 'N';
        integer m = B.size(0), n = B.size(1),
                lda = A.gdim(0), ldb = B.gdim(0);

  F77NAME(dtrsm)(&side, &uplo, &transa, &diag, &m, &n, &alpha,
                &A(0,0), &lda, &B(0,0), &ldb);
}

#endif

#ifdef _LA_SYMM_MAT_DOUBLE_H_
void Blas_Mat_Mat_Mult(LaSymmMatDouble &A, LaGenMatDouble &B, 
            LaGenMatDouble &C, double alpha , double beta )
{
        char side = 'L', uplo = 'L';
        integer m = C.size(0), n = C.size(1), lda = A.gdim(0), 
                ldb = B.gdim(0), ldc = C.gdim(0);

  F77NAME(dsymm)(&side, &uplo, &m, &n, &alpha, &A(0,0), &lda, 
                &B(0,0), &ldb, &beta, &C(0,0), &ldc);
}

void Blas_R1_Update(LaSymmMatDouble &C, LaGenMatDouble &A,
            double alpha , double beta )
{
        char uplo = 'L', transa = 'N';
        integer n = C.size(0), k = A.size(1), 
                lda = A.gdim(0), ldc = C.gdim(0);

  F77NAME(dsyrk)(&uplo, &transa, &n, &k, &alpha, &A(0,0), &lda, 
                &beta, &C(0,0), &ldc);
}


void Blas_R2_Update(LaSymmMatDouble &C, LaGenMatDouble &A,
            LaGenMatDouble &B, double alpha , double beta )
{
        char uplo = 'L', transa = 'N';
        integer n = C.size(0), k = A.size(1), lda = A.gdim(0), 
                ldb = B.gdim(0), ldc = C.gdim(0);

  F77NAME(dsyr2k)(&uplo, &transa, &n, &k, &alpha, &A(0,0), &lda, 
                &B(0,0), &ldb, &beta, &C(0,0), &ldc);
}

#endif


void Blas_Mat_Mat_Mult(const LaGenMatComplex &A, 
            const LaGenMatComplex &B, LaGenMatComplex &C, 
            LaComplex _alpha, LaComplex _beta)
{
        char t = 'N';
        integer m = A.size(0), k = A.size(1), n = B.size(1);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(0) == C.size(0));
	assert(B.size(1) == C.size(1));
	assert(A.size(1) == B.size(0));
    doublecomplex alpha(_alpha);
    doublecomplex beta(_beta);

  F77NAME(zgemm)(&t, &t, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}

void Blas_Mat_Trans_Mat_Mult(const LaGenMatComplex &A, 
            const LaGenMatComplex &B, LaGenMatComplex &C, 
            LaComplex _alpha, LaComplex _beta)
{
        char nt = 'N';
        char t = 'C';
	// m, n, k have to be determined according to op(A), op(B)!
        integer m = A.size(1), k = A.size(0), n = B.size(1);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(1) == C.size(0));
	assert(B.size(1) == C.size(1));
	assert(A.size(0) == B.size(0));
    doublecomplex alpha(_alpha);
    doublecomplex beta(_beta);

  F77NAME(zgemm)(&t, &nt, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}

void Blas_Mat_Mat_Trans_Mult(const LaGenMatComplex &A, 
            const LaGenMatComplex &B, LaGenMatComplex &C, 
            LaComplex _alpha, LaComplex _beta)
{
        char nt = 'N';
        char t = 'C';
	// m, n, k have to be determined according to op(A), op(B)!
        integer m = A.size(0), k = A.size(1), n = B.size(0);
        integer lda = A.gdim(0), ldb = B.gdim(0), ldc = C.gdim(0);
	assert(A.size(0) == C.size(0));
	assert(B.size(0) == C.size(1));
	assert(A.size(1) == B.size(1));
    doublecomplex alpha(_alpha);
    doublecomplex beta(_beta);

  F77NAME(zgemm)(&nt, &t, &m, &n, &k, &alpha, &A(0,0), &lda, &B(0,0), 
                &ldb, &beta, &C(0,0), &ldc);
    
}

// Matrix norms

double Blas_Norm1(const LaGenMatDouble &A)
{
    integer M=A.size(1);

    // max column-sum

    if (M==1)
    {
       return Blas_Norm1( LaVectorDouble( A ) );
    }
    else
    {
       LaVectorDouble R(M);

       for (integer i=0; i<M; i++)
	  R(i) = Blas_Norm1( LaVectorDouble( A( LaIndex(), i )) );

       return Blas_Norm_Inf(R);
    }
}

double Blas_Norm1(const LaGenMatComplex &A)
{
    integer M=A.size(1);

    // max column-sum

    if (M==1)
    {
       return Blas_Norm1( LaVectorComplex( A ) );
    }
    else
    {
       LaVectorDouble R(M);

       for (integer i=0; i<M; i++)
	  R(i) = Blas_Norm1( LaVectorComplex( A( LaIndex(), i )) );
       
       return Blas_Norm_Inf(R);
    }
}

// max row norm

double Blas_Norm_Inf(const LaGenMatDouble &A)
{
    integer M=A.size(0);

    // max row-sum

    LaVectorDouble R(M);

    for (integer i=0; i<M; i++)
        R(i) = Blas_Norm1( LaVectorDouble( A( i, LaIndex() )) );

    return Blas_Norm_Inf(R);
}

double Blas_Norm_Inf(const LaGenMatComplex &A)
{
    integer M=A.size(0);

    // max row-sum

    LaVectorDouble R(M);

    for (integer i=0; i<M; i++)
        R(i) = Blas_Norm1( LaVectorComplex( A( i, LaIndex() )) );

    return Blas_Norm_Inf(R);
}

double Blas_NormF(const LaGenMatComplex &A)
{
   return Blas_Norm2( LaVectorComplex( &A(0,0), A.size(0)*A.size(1) ) );
}
double Blas_NormF(const LaGenMatDouble &A)
{
   return Blas_Norm2( LaVectorDouble( &A(0,0), A.size(0)*A.size(1) ) );
}

/** DEPRECATED, use Blas_Norm_Inf instead. */
double Norm_Inf(const LaGenMatDouble &A) 
{ return Blas_Norm_Inf(A); }
/** DEPRECATED, use Blas_Norm_Inf instead. */
double Norm_Inf(const LaGenMatComplex &A) 
{ return Blas_Norm_Inf(A); }
